#ifndef ROGUE_WORLD_H
#define ROGUE_WORLD_H

#include <optional>
#include <vector>
#include <SDL2/SDL_rect.h>
#include "character.h"
#include "core/input.h"
#include "core/roguedefs.h"

namespace rogue {
class Character;

struct TileData {
	Character *character{nullptr};
	bool is_wall{false};
};

struct Room {
	// bitmask of hallways
	Directions hallway_paths{0};
	int chunk_size{0};
	bool initialized{false};
	// the starting tile and length of the walls
	SDL_Rect rect{0, 0, 1, 1};
	void draw(Tile pivot);
	bool tile_is_wall(Tile local_tile) const;
};

struct World {
	friend class WorldGenerator;
	World();
	~World() = default;
	void act();
	void render();
	Room &get_room(Chunk chunk);
	TileData query_tile(Tile tile);
	void on_input(bool);
private:
	int chunk_size{1};
	unsigned shear{0};
	Character *player{nullptr};
	std::vector<Room> rooms{};
	std::vector<Character> characters{};
	KeyEventHandler direction_pressed;
private:
};

class WorldGenerator {
public:
	WorldGenerator() = default;
	WorldGenerator &with_chunk_size(unsigned side_length);
	WorldGenerator &with_world_size(unsigned side_length);
	WorldGenerator &with_room_size(unsigned min_side_length, unsigned max_side_length);
	WorldGenerator &with_player(Character character);
	std::unique_ptr<World> generate();
private:
	Chunk select_start() const;
	void generate_world(std::unique_ptr<World> &world, Chunk start);
	void generate_room(std::unique_ptr<World> &world, Chunk last);
	void setup_player(std::unique_ptr<World> &world, Chunk spawn_chunk);
private:
	std::optional<Character> player;
	int chunk_side_length{0}, world_side_length{0};
	int max_room_size{0}, min_room_size{0};
};
}

#endif // !ROGUE_WORLD_H
