#include <SDL2/SDL.h>
#include <memory>
#include "core/character.h"
#include "core/input.h"
#include "core/renderer.h"
#include "core/world.h"
#include "player_logic.h"

using namespace rogue;

int main(int argc, char *argv[]) {
	std::unique_ptr<World> world{WorldGenerator()
		.with_world_size(5)
		.with_chunk_size(9)
		.with_room_size(2, 5)
		.with_player(Character({9/2, 9/2},
			new PlayerCharacterLogic(),
			CharacterData {
				.health = 1,
				.damage = 1,
				.sprite = 0
			}
		))
		.generate()
	};
	RenderData data{RenderDataSetup()
		.with_window("roguelike", SDL_WINDOW_ALLOW_HIGHDPI | SDL_WINDOW_FULLSCREEN)
		.with_renderer(SDL_RENDERER_ACCELERATED)
		.with_resource_path("resources/")
		.with_sprite("wizard.png")
		.with_sprite("wall.png")
		.build()
	};
	Render::provide_render_data(data);
	SDL_Event evt;
	for(;;) {
		world->render();
		Render::present();
		if(SDL_WaitEvent(&evt)) {
			if(evt.type == SDL_QUIT) {
				goto main_loop;
			} else {
				Input::process_event(evt);
			}
		}
	} main_loop:
	return 0;
}
